import string

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``sk_SK`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Slovakia
    """

    license_plate_prefix = [
        'BA', 'BL', 'BT',  # Bratislava
        'BB',              # Banska Bystrica
        'BJ',              # Bardejov
        'BN',              # Banovce nad Bebravou
        'BR',              # Brezno
        'BS',              # Banska Stiavnica
        'BY',              # Bytca
        'CA',              # Cadca
        'DK',              # Dolny Kubin
        'DS',              # Dunajska Streda
        'DT',              # Detva
        'GA',              # Galanta
        'GL',              # Gelnica
        'HC',              # Hlohovec
        'HE',              # Humenne
        'IL',              # Ilava
        'KA',              # Krupina
        'KE',              # Kosice
        'KK',              # Kezmarok
        'KM',              # Kysucke Nove Mesto
        'KN',              # Komarno
        'KS',              # Kosice-okolie
        'LC',              # Lucenec
        'LE',              # Levoca
        'LM',              # Liptovsky Mikulas
        'LV',              # Levice
        'MA',              # Malacky
        'MI',              # Michalovce
        'ML',              # Medzilaborce
        'MT',              # Martin
        'MY',              # Myjava
        'NR',              # Nitra
        'NM',              # Nove Mesto nad Vahom
        'NO',              # Namestovo
        'NZ',              # Nove Zamky
        'PB',              # Povazska Bystrica
        'PD',              # Prievidza
        'PE',              # Partizanske
        'PK',              # Pezinok
        'PN',              # Piestany
        'PO',              # Presov
        'PP',              # Poprad
        'PT',              # Poltar
        'PU',              # Puchov
        'RA',              # Revuca
        'RK',              # Ruzomberok
        'RS',              # Rimavska Sobota
        'RV',              # Roznava
        'SA',              # Sala
        'SB',              # Sabinov
        'SC',              # Senec
        'SE',              # Senica
        'SI',              # Skalica
        'SK',              # Svidnik
        'SL',              # Stara Lubovna
        'SN',              # Spisska Nova Ves
        'SO',              # Sobrance
        'SP',              # Stropkov
        'SV',              # Snina
        'TT',              # Trnava
        'TN',              # Trencin
        'TO',              # Topolcany
        'TR',              # Turcianske Teplice
        'TS',              # Tvrdosin
        'TV',              # Trebisov
        'VK',              # Velky Krtis
        'VT',              # Vranov nad Toplou
        'ZA',              # Zilina
        'ZC',              # Zarnovica
        'ZH',              # Ziar nad Hronom
        'ZM',              # Zlate Moravce
        'ZV',              # Zvolen
    ]

    license_plate_suffix = (
        '###??',
    )

    def license_plate(self):
        """Generate a license plate."""
        prefix = self.random_element(self.license_plate_prefix)
        suffix = self.bothify(self.random_element(self.license_plate_suffix), letters=string.ascii_uppercase)
        return prefix + suffix



import string

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``ro_RO`` locale. """

    license_plate_prefix = (
        'AB', 'AG', 'AR', 'B', 'BC', 'BH', 'BN', 'BR', 'BT', 'BV', 'BZ',
        'CJ', 'CL', 'CS', 'CT', 'CV', 'DB', 'DJ', 'GJ', 'GL', 'GR', 'HD',
        'HR', 'IF', 'IL', 'IS', 'MH', 'MM', 'MS', 'NT', 'OT', 'PH', 'SB',
        'SJ', 'SM', 'SV', 'TL', 'TM', 'TR', 'VL', 'VN', 'VS',
    )

    license_plate_suffix = (
        '-###-???',
        '-##-???',
    )

    def license_plate(self):
        """Generate a license plate."""
        prefix = self.random_element(self.license_plate_prefix)
        suffix = self.bothify(self.random_element(
            self.license_plate_suffix), letters=string.ascii_uppercase)
        return prefix + suffix



import csv
import hashlib
import io
import json
import re
import string
import tarfile
import uuid
import zipfile

from .. import BaseProvider

localized = True

csv.register_dialect('faker-csv', csv.excel, quoting=csv.QUOTE_ALL)


class Provider(BaseProvider):

    def boolean(self, chance_of_getting_true=50):
        """Generate a random boolean value based on ``chance_of_getting_true``.

        :sample size=10: chance_of_getting_true=25
        :sample size=10: chance_of_getting_true=50
        :sample size=10: chance_of_getting_true=75
        """
        return self.generator.random.randint(1, 100) <= chance_of_getting_true

    def null_boolean(self):
        """Generate ``None``, ``True``, or ``False``, each with equal probability.

        :sample size=15:
        """
        return {
            0: None,
            1: True,
            -1: False,
        }[self.generator.random.randint(-1, 1)]

    def binary(self, length=(1 * 1024 * 1024)):
        """Generate a random binary blob of ``length`` bytes.

        :sample: length=64
        """
        blob = [self.generator.random.randrange(256) for _ in range(length)]
        return bytes(blob)

    def md5(self, raw_output=False):
        """Generate a random MD5 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the MD5 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res = hashlib.md5(str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def sha1(self, raw_output=False):
        """Generate a random SHA1 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the SHA1 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res = hashlib.sha1(str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def sha256(self, raw_output=False):
        """Generate a random SHA256 hash.

        If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the SHA56 hash
        will be returned. If ``True``, a ``bytes`` object representation will be returned instead.

        :sample: raw_output=False
        :sample: raw_output=True
        """
        res = hashlib.sha256(
            str(self.generator.random.random()).encode())
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def uuid4(self, cast_to=str):
        """Generate a random UUID4 object and cast it to another type if specified using a callable ``cast_to``.

        By default, ``cast_to`` is set to ``str``.

        May be called with ``cast_to=None`` to return a full-fledged ``UUID``.

        :sample:
        :sample: cast_to=None
        """
        # Based on http://stackoverflow.com/q/41186818
        generated_uuid = uuid.UUID(int=self.generator.random.getrandbits(128), version=4)
        if cast_to is not None:
            generated_uuid = cast_to(generated_uuid)
        return generated_uuid

    def password(
            self,
            length=10,
            special_chars=True,
            digits=True,
            upper_case=True,
            lower_case=True):
        """Generate a random password of the specified ``length``.

        The arguments ``special_chars``, ``digits``, ``upper_case``, and ``lower_case`` control
        what category of characters will appear in the generated password. If set to ``True``
        (default), at least one character from the corresponding category is guaranteed to appear.
        Special characters are characters from ``!@#$%^&*()_+``, digits are characters from
        ``0123456789``, and uppercase and lowercase characters are characters from the ASCII set of
        letters.

        :sample: length=12
        :sample: length=40, special_chars=False, upper_case=False
        """
        choices = ""
        required_tokens = []
        if special_chars:
            required_tokens.append(
                self.generator.random.choice("!@#$%^&*()_+"))
            choices += "!@#$%^&*()_+"
        if digits:
            required_tokens.append(self.generator.random.choice(string.digits))
            choices += string.digits
        if upper_case:
            required_tokens.append(
                self.generator.random.choice(string.ascii_uppercase))
            choices += string.ascii_uppercase
        if lower_case:
            required_tokens.append(
                self.generator.random.choice(string.ascii_lowercase))
            choices += string.ascii_lowercase

        assert len(
            required_tokens) <= length, "Required length is shorter than required characters"

        # Generate a first version of the password
        chars = self.random_choices(choices, length=length)

        # Pick some unique locations
        random_indexes = set()
        while len(random_indexes) < len(required_tokens):
            random_indexes.add(
                self.generator.random.randint(0, len(chars) - 1))

        # Replace them with the required characters
        for i, index in enumerate(random_indexes):
            chars[index] = required_tokens[i]

        return ''.join(chars)

    def zip(self, uncompressed_size=65536, num_files=1, min_file_size=4096, compression=None):
        """Generate a bytes object containing a random valid zip archive file.

        The number and sizes of files contained inside the resulting archive can be controlled
        using the following arguments:

        - ``uncompressed_size`` - the total size of files before compression, 16 KiB by default
        - ``num_files`` - the number of files archived in resulting zip file, 1 by default
        - ``min_file_size`` - the minimum size of each file before compression, 4 KiB by default

        No compression is used by default, but setting ``compression`` to one of the values listed
        below will use the corresponding compression type.

        - ``'bzip2'`` or ``'bz2'`` for BZIP2
        - ``'lzma'`` or ``'xz'`` for LZMA
        - ``'deflate'``, ``'gzip'``, or ``'gz'`` for GZIP

        :sample: uncompressed_size=256, num_files=4, min_file_size=32
        :sample: uncompressed_size=256, num_files=32, min_file_size=4, compression='bz2'
        """
        if any([
            not isinstance(num_files, int) or num_files <= 0,
            not isinstance(min_file_size, int) or min_file_size <= 0,
            not isinstance(uncompressed_size, int) or uncompressed_size <= 0,
        ]):
            raise ValueError(
                '`num_files`, `min_file_size`, and `uncompressed_size` must be positive integers',
            )
        if min_file_size * num_files > uncompressed_size:
            raise AssertionError(
                '`uncompressed_size` is smaller than the calculated minimum required size',
            )
        if compression in ['bzip2', 'bz2']:
            compression = zipfile.ZIP_BZIP2
        elif compression in ['lzma', 'xz']:
            compression = zipfile.ZIP_LZMA
        elif compression in ['deflate', 'gzip', 'gz']:
            compression = zipfile.ZIP_DEFLATED
        else:
            compression = zipfile.ZIP_STORED

        zip_buffer = io.BytesIO()
        remaining_size = uncompressed_size
        with zipfile.ZipFile(zip_buffer, mode='w', compression=compression) as zip_handle:
            for file_number in range(1, num_files + 1):
                filename = self.generator.pystr() + str(file_number)

                max_allowed_size = remaining_size - (num_files - file_number) * min_file_size
                if file_number < num_files:
                    file_size = self.generator.random.randint(min_file_size, max_allowed_size)
                    remaining_size = remaining_size - file_size
                else:
                    file_size = remaining_size

                data = self.generator.binary(file_size)
                zip_handle.writestr(filename, data)
        return zip_buffer.getvalue()

    def tar(self, uncompressed_size=65536, num_files=1, min_file_size=4096, compression=None):
        """Generate a bytes object containing a random valid tar file.

        The number and sizes of files contained inside the resulting archive can be controlled
        using the following arguments:

        - ``uncompressed_size`` - the total size of files before compression, 16 KiB by default
        - ``num_files`` - the number of files archived in resulting zip file, 1 by default
        - ``min_file_size`` - the minimum size of each file before compression, 4 KiB by default

        No compression is used by default, but setting ``compression`` to one of the values listed
        below will use the corresponding compression type.

        - ``'bzip2'`` or ``'bz2'`` for BZIP2
        - ``'lzma'`` or ``'xz'`` for LZMA
        - ``'gzip'`` or ``'gz'`` for GZIP

        :sample: uncompressed_size=256, num_files=4, min_file_size=32
        :sample: uncompressed_size=256, num_files=32, min_file_size=4, compression='bz2'
        """
        if any([
            not isinstance(num_files, int) or num_files <= 0,
            not isinstance(min_file_size, int) or min_file_size <= 0,
            not isinstance(uncompressed_size, int) or uncompressed_size <= 0,
        ]):
            raise ValueError(
                '`num_files`, `min_file_size`, and `uncompressed_size` must be positive integers',
            )
        if min_file_size * num_files > uncompressed_size:
            raise AssertionError(
                '`uncompressed_size` is smaller than the calculated minimum required size',
            )
        if compression in ['gzip', 'gz']:
            mode = 'w:gz'
        elif compression in ['bzip2', 'bz2']:
            mode = 'w:bz2'
        elif compression in ['lzma', 'xz']:
            mode = 'w:xz'
        else:
            mode = 'w'

        tar_buffer = io.BytesIO()
        remaining_size = uncompressed_size
        with tarfile.open(mode=mode, fileobj=tar_buffer) as tar_handle:
            for file_number in range(1, num_files + 1):
                file_buffer = io.BytesIO()
                filename = self.generator.pystr() + str(file_number)

                max_allowed_size = remaining_size - (num_files - file_number) * min_file_size
                if file_number < num_files:
                    file_size = self.generator.random.randint(min_file_size, max_allowed_size)
                    remaining_size = remaining_size - file_size
                else:
                    file_size = remaining_size

                tarinfo = tarfile.TarInfo(name=filename)
                data = self.generator.binary(file_size)
                file_buffer.write(data)
                tarinfo.size = len(file_buffer.getvalue())
                file_buffer.seek(0)
                tar_handle.addfile(tarinfo, file_buffer)
                file_buffer.close()
        return tar_buffer.getvalue()

    def dsv(self, dialect='faker-csv', header=None,
            data_columns=('{{name}}', '{{address}}'),
            num_rows=10, include_row_ids=False, **fmtparams):
        """Generate random delimiter-separated values.

        This method's behavior share some similarities with ``csv.writer``. The ``dialect`` and
        ``**fmtparams`` arguments are the same arguments expected by ``csv.writer`` to control its
        behavior, and instead of expecting a file-like object to where output will be written, the
        output is controlled by additional keyword arguments and is returned as a string.

        The ``dialect`` argument defaults to ``'faker-csv'`` which is the name of a ``csv.excel``
        subclass with full quoting enabled.

        The ``header`` argument expects a list or a tuple of strings that will serve as the header row
        if supplied. The ``data_columns`` argument expects a list or a tuple of string tokens, and these
        string tokens will be passed to  :meth:`pystr_format() <faker.providers.python.Provider.pystr_format>`
        for data generation. Argument Groups are used to pass arguments to the provider methods.
        Both ``header`` and ``data_columns`` must be of the same length.

        Example:
            fake.set_arguments('top_half', {'min_value': 50, 'max_value': 100})
            fake.dsv(data_columns=('{{ name }}', '{{ pyint:top_half }}'))

        The ``num_rows`` argument controls how many rows of data to generate, and the ``include_row_ids``
        argument may be set to ``True`` to include a sequential row ID column.

        :sample: dialect='excel', data_columns=('{{name}}', '{{address}}')
        :sample: dialect='excel-tab', data_columns=('{{name}}', '{{address}}'), include_row_ids=True
        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=5, delimiter='$'
        """

        if not isinstance(num_rows, int) or num_rows <= 0:
            raise ValueError('`num_rows` must be a positive integer')
        if not isinstance(data_columns, (list, tuple)):
            raise TypeError('`data_columns` must be a tuple or a list')
        if header is not None:
            if not isinstance(header, (list, tuple)):
                raise TypeError('`header` must be a tuple or a list')
            if len(header) != len(data_columns):
                raise ValueError('`header` and `data_columns` must have matching lengths')

        dsv_buffer = io.StringIO()
        writer = csv.writer(dsv_buffer, dialect=dialect, **fmtparams)

        if header:
            if include_row_ids:
                header = list(header)
                header.insert(0, 'ID')
            writer.writerow(header)

        for row_num in range(1, num_rows + 1):
            row = [self.generator.pystr_format(column) for column in data_columns]
            if include_row_ids:
                row.insert(0, str(row_num))

            writer.writerow(row)

        return dsv_buffer.getvalue()

    def csv(self, header=None, data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False):
        """Generate random comma-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header, data_columns=data_columns, num_rows=num_rows,
            include_row_ids=include_row_ids, delimiter=',',
        )

    def tsv(self, header=None, data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False):
        """Generate random tab-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header, data_columns=data_columns, num_rows=num_rows,
            include_row_ids=include_row_ids, delimiter='\t',
        )

    def psv(self, header=None, data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False):
        """Generate random pipe-separated values.

        For more information on the different arguments of this method, please refer to
        :meth:`dsv() <faker.providers.misc.Provider.dsv>` which is used under the hood.

        :sample: data_columns=('{{name}}', '{{address}}'), num_rows=10, include_row_ids=False
        :sample: header=('Name', 'Address', 'Favorite Color'),
                data_columns=('{{name}}', '{{address}}', '{{safe_color_name}}'),
                num_rows=10, include_row_ids=True
        """
        return self.dsv(
            header=header, data_columns=data_columns, num_rows=num_rows,
            include_row_ids=include_row_ids, delimiter='|',
        )

    def json(self,
             data_columns: list = None,
             num_rows: int = 10,
             indent: int = None) -> str:
        """
        Generate random JSON structure values.

        Using a dictionary or list of records that is passed as ``data_columns``,
        define the structure that is used to build JSON structures.  For complex
        data structures it is recommended to use the dictionary format.

        Data Column Dictionary format:
            {'key name': 'definition'}

        The definition can be 'provider', 'provider:argument_group', tokenized
        'string {{ provider:argument_group }}' that is passed to the python
        provider method pystr_format() for generation, or a fixed '@word'.
        Using Lists, Tuples, and Dicts as a definition for structure.

        Example:
            fake.set_arguments('top_half', {'min_value': 50, 'max_value': 100})
            fake.json(data_columns={'Name': 'name', 'Score': 'pyint:top_half'})

        Data Column List format:
            [('key name', 'definition', {'arguments'})]

        With the list format the definition can be a list of records, to create
        a list within the structure data.  For literal entries within the list,
        set the 'field_name' to None.

        :param data_columns: specification for the data structure
        :type data_columns: dict
        :param num_rows: number of rows the returned
        :type num_rows: int
        :param indent: number of spaces to indent the fields
        :type indent: int
        :return: Serialized JSON data
        :rtype: str

        :sample: data_columns={'Spec': '@1.0.1', 'ID': 'pyint',
                'Details': {'Name': 'name', 'Address': 'address'}}, num_rows=2
        :sample: data_columns={'Candidates': ['name', 'name', 'name']},
                num_rows=1
        :sample: data_columns=[('Name', 'name'), ('Points', 'pyint',
                {'min_value': 50, 'max_value': 100})], num_rows=1
        """
        default_data_columns = {
            'name': '{{name}}',
            'residency': '{{address}}',
        }
        data_columns = data_columns if data_columns else default_data_columns

        def process_list_structure(data: list) -> dict:
            entry = {}

            for name, definition, *arguments in data:
                kwargs = arguments[0] if arguments else {}

                if not isinstance(kwargs, dict):
                    raise TypeError('Invalid arguments type. Must be a dictionary')

                if name is None:
                    return self._value_format_selection(definition, **kwargs)

                if isinstance(definition, tuple):
                    entry[name] = process_list_structure(definition)
                elif isinstance(definition, (list, set)):
                    entry[name] = [process_list_structure([item])
                                   for item in definition]
                else:
                    entry[name] = self._value_format_selection(definition, **kwargs)
            return entry

        def process_dict_structure(data: dict) -> dict:
            entry = {}

            if isinstance(data, str):
                return self._value_format_selection(data)

            if isinstance(data, dict):
                for name, definition in data.items():
                    if isinstance(definition, (tuple, list, set)):
                        entry[name] = [process_dict_structure(item)
                                       for item in definition]
                    elif isinstance(definition, (dict, int, float, bool)):
                        entry[name] = process_dict_structure(definition)
                    else:
                        entry[name] = self._value_format_selection(definition)
                return entry

            return data

        def create_json_structure(data_columns) -> dict:
            if isinstance(data_columns, dict):
                return process_dict_structure(data_columns)

            if isinstance(data_columns, list):
                return process_list_structure(data_columns)

            raise TypeError('Invalid data_columns type. Must be a dictionary or list')

        if num_rows == 1:
            return json.dumps(create_json_structure(data_columns), indent=indent)

        data = [create_json_structure(data_columns) for _ in range(num_rows)]
        return json.dumps(data, indent=indent)

    def fixed_width(self,
                    data_columns: list = None,
                    num_rows: int = 10,
                    align: str = 'left') -> str:
        """
        Generate random fixed width values.

        Using a list of tuple records that is passed as ``data_columns``, that
        defines the structure that will be generated. Arguments within the
        record are provider specific, and should be a dictionary that will be
        passed to the provider method.

        Data Column List format
            [('field width', 'definition', {'arguments'})]

        The definition can be 'provider', 'provider:argument_group', tokenized
        'string {{ provider:argument_group }}' that is passed to the python
        provider method pystr_format() for generation, or a fixed '@word'.
        Using Lists, Tuples, and Dicts as a definition for structure.

        Argument Groups can be used to pass arguments to the provider methods,
        but will override the arguments supplied in the tuple record.

        Example:
            fake.set_arguments('top_half', {'min_value': 50, 'max_value': 100})
            fake.fixed_width(data_columns=[(20, 'name'), (3, 'pyint:top_half')])

        :param data_columns: specification for the data structure
        :type data_columns: list
        :param num_rows: number of rows the generator will yield
        :type num_rows: int
        :param align: positioning of the value. (left, middle, right)
        :type align: str
        :return: Serialized Fixed Width data
        :rtype: str

        :sample: data_columns=[(20, 'name'), (3, 'pyint', {'min_value': 50,
                'max_value': 100})], align='right', num_rows=2
        """
        default_data_columns = [
            (20, 'name'),
            (3, 'pyint', {'max_value': 20}),
        ]
        data_columns = data_columns if data_columns else default_data_columns
        align_map = {
            'left': '<',
            'middle': '^',
            'right': '>',
        }
        data = []

        for _ in range(num_rows):
            row = []

            for width, definition, *arguments in data_columns:
                kwargs = arguments[0] if arguments else {}

                if not isinstance(kwargs, dict):
                    raise TypeError('Invalid arguments type. Must be a dictionary')

                result = self._value_format_selection(definition, **kwargs)
                row.append(f'{result:{align_map.get(align, "<")}{width}}'[:width])

            data.append(''.join(row))
        return '\n'.join(data)

    def _value_format_selection(self, definition, **kwargs):
        """
        Formats the string in different ways depending on it's contents.

        The return can be the '@word' itself, a '{{ token }}' passed to PyStr,
        or a 'provider:argument_group' format field that returns potentially
        a non-string type.

        This ensures that Numbers, Boolean types that are generated in the
        JSON structures in there proper type, and not just strings.
        """

        # Check for PyStr first as complex strings may start with @
        if re.match(r'.*\{\{.*\}\}.*', definition):
            return self.generator.pystr_format(definition)

        # Check for fixed @words that won't be generated
        if re.match(r'^@.*', definition):
            return definition.lstrip('@')

        # Check if a argument group has been supplied
        if re.match(r'^[a-zA-Z0-9_-]*:\w', definition):
            definition, argument_group = definition.split(':')
            arguments = self.generator.get_arguments(argument_group.strip())

            return self.generator.format(definition.strip(), **arguments)

        # Assume the string is refering to a provider
        return self.generator.format(definition, **kwargs)



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):

    price_formats = ["#,##", "%#,##", "%##,##", "%.###,##", "%#.###,##"]

    def pricetag(self):
        return (
            self.numerify(self.random_element(self.price_formats))
            + "\N{no-break space}zł"
        )



from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        'gmail.com', 'googlemail.com', 'hotmail.com', 'suomi24.fi',
        'kolumbus.fi', 'luukku.com', 'surffi.net',
    )

    tlds = ('com', 'com', 'com', 'fi', 'fi', 'net', 'org')



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    city_formats = ('{{canton_name}}',)
    building_number_formats = ('%', '%#', '%#', '%#', '%##')
    street_suffixes = ['strasse']
    street_name_formats = ('{{last_name}}{{street_suffix}}', )
    street_address_formats = ('{{street_name}} {{building_number}}', )
    address_formats = ('{{street_address}}\n{{postcode}} {{city}}', )
    postcode_formats = ('1###', '2###', '3###', '4###', '5###', '6###', '7###',
                        '8###', '9###')

    cantons = (('AG', 'Aargau'), ('AI', 'Appenzell Innerrhoden'),
               ('AR', 'Appenzell Ausserrhoden'), ('BE', 'Bern'),
               ('BL', 'Basel-Landschaft'), ('BS', 'Basel-Stadt'), ('FR', 'Freiburg'),
               ('GE', 'Genf'), ('GL', 'Glarus'), ('GR', 'Graubünden'), ('JU', 'Jura'),
               ('LU', 'Luzern'), ('NE', 'Neuenburg'), ('NW', 'Nidwalden'), ('OW', 'Obwalden'),
               ('SG', 'St. Gallen'), ('SH', 'Schaffhausen'), ('SO', 'Solothurn'),
               ('SZ', 'Schwyz'), ('TG', 'Thurgau'), ('TI', 'Tessin'), ('UR', 'Uri'),
               ('VD', 'Waadt'), ('VS', 'Wallis'), ('ZG', 'Zug'), ('ZH', 'Zürich'))

    def canton(self):
        """
        Randomly returns a swiss canton ('Abbreviated', 'Name').
        :example ('ZH', 'Zürich')
        """
        return self.random_element(self.cantons)

    def administrative_unit(self):
        """
        Randomly returns a Swiss canton name.
        :example 'Zürich'
        """
        return self.canton()[1]

    canton_name = administrative_unit

    def canton_code(self):
        """
        Randomly returns a Swiss canton code.
        :example 'ZH'
        """
        return self.canton()[0]



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``en_CA`` locale.

    Sources:

    - https://www.revolvy.com/main/index.php?s=Canadian%20licence%20plate%20designs%20and%20serial%20formats
    """

    license_formats = (
        # Alberta
        '???-####',
        # BC
        '??# ##?',
        '?? ####',
        # Manitoba
        '??? ###',
        # New Brunswick
        '??? ###',
        # Newfoundland and Labrador
        '??? ###',
        # NWT
        '######',
        # Nova Scotia
        '??? ###',
        # Nunavut
        '### ###',
        # Ontario
        '### ???',
        '???? ###',
        '??# ###',
        '### #??',
        '?? ####',
        'GV??-###',
        # PEI
        '## ##??',
        # Quebec
        '?## ???',
        # Saskatchewan
        '### ???',
        # Yukon
        '???##',
    )



from .. import Provider as BaseProvider


def ssn_checksum(number):
    """
    Calculate the checksum for the romanian SSN (CNP).
    """
    weights = (2, 7, 9, 1, 4, 6, 3, 5, 8, 2, 7, 9)
    check = sum(w * int(n) for w, n in zip(weights, number)) % 11
    return 1 if check == 10 else check


def vat_checksum(number):
    """
    Calculate the check digit for romanian VAT numbers.
    """
    weights = (7, 5, 3, 2, 1, 7, 5, 3, 2)
    number = (9 - len(number)) * '0' + number
    check = 10 * sum(w * int(n) for w, n in zip(weights, number))
    return check % 11 % 10


class Provider(BaseProvider):
    """
    A Faker provider for the Romanian VAT IDs
    """

    vat_id_formats = (
        'RO1########',
        'RO2########',
        'RO3########',
        'RO4########',
        'RO5########',
        'RO6########',
        'RO7########',
        'RO8########',
        'RO9########',
        '1########',
        '2########',
        '3########',
        '4########',
        '5########',
        '6########',
        '7########',
        '8########',
        '9########',
    )

    def vat_id(self):
        """
        https://ro.wikipedia.org/wiki/Cod_de_identificare_fiscal%C4%83
        :return: A random Romanian VAT ID
        """
        vat = self.bothify(self.random_element(self.vat_id_formats))
        coutry = ""
        if vat.startswith('RO'):
            coutry = 'RO'
            vat = vat[2:]
        check = vat_checksum(vat)
        vat += str(check)
        return coutry + vat

    ssn_formats = ("#############",)

    def ssn(self):
        """
        Romanian Social Security Number.

        :return: a random Romanian SSN
        """
        gender = self.random_int(min=1, max=8)
        year = self.random_int(min=0, max=99)
        month = self.random_int(min=1, max=12)
        day = self.random_int(min=1, max=31)
        county = int(
            self.random_element([
                '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12',
                '13', '14', '15', '16', '17', '18', '19', '20', '21', '22',
                '23', '24', '25', '26', '27', '28', '29', '30', '31', '32',
                '33', '34', '35', '36', '37', '38', '39', '40', '41', '42',
                '43', '44', '45', '46', '51', '52',
            ]))
        serial = self.random_int(min=1, max=999)

        num = f'{gender:01d}{year:02d}{month:02d}{day:02d}{county:02d}{serial:03d}'

        check = ssn_checksum(num)
        num += str(check)
        return num



from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``cs_CZ`` locale.

    Word list is drawn from the SYN2015.
    (representative corpus of contemporary written Czech published in December 2015)

    The word list is a list of the 2000 most common lemmas. Abbreviations and first names were removed.

    Sources:
    - https://wiki.korpus.cz/lib/exe/fetch.php/seznamy:syn2015_lemma_utf8.zip

    """

    word_list = (
        'a', 'aby', 'adresa', 'Afrika', 'agentura', 'akce', 'aktivita', 'aktivní', 'aktuální', 'ale', 'alespoň',
        'alkohol', 'americký', 'Amerika', 'analýza', 'anebo', 'anglický', 'ani', 'aniž', 'ano', 'aplikace', 'architekt',
        'areál', 'armáda', 'asi', 'aspoň', 'atmosféra', 'auto', 'autobus', 'autor', 'avšak', 'ačkoli', 'ať', 'až',
        'babička', 'banka', 'barevný', 'barva', 'bavit', 'bez', 'bezpečnost', 'bezpečnostní', 'bezpečný', 'blok',
        'blízko', 'blízký', 'blížit', 'bod', 'bohatý', 'bohužel', 'boj', 'bojovat', 'bok', 'bolest', 'bota', 'boží',
        'branka', 'bratr', 'britský', 'Brno', 'brněnský', 'brzy', 'brána', 'bránit', 'brát', 'budoucnost', 'budoucí',
        'budova', 'buď', 'buňka', 'bydlet', 'byt', 'byť', 'bát', 'bílý', 'být', 'bývalý', 'bývat', 'během', 'běžet',
        'běžný', 'břeh', 'březen', 'břicho', 'bůh', 'celek', 'celkem', 'celkový', 'celý', 'cena', 'centrum', 'cesta',
        'charakter', 'chladný', 'chlap', 'chlapec', 'chodba', 'chodit', 'chovat', 'chování', 'chránit', 'chtít', 'chuť',
        'chvilka', 'chvíle', 'chyba', 'chybět', 'chystat', 'chytit', 'chápat', 'cigareta', 'cizí', 'co', 'cokoli',
        'cosi', 'což', 'cukr', 'cíl', 'církev', 'cítit', 'daleko', 'další', 'daný', 'datum', 'daň', 'dařit', 'dcera',
        'dech', 'den', 'denně', 'deník', 'deset', 'design', 'deska', 'desítka', 'detail', 'devět', 'diskuse', 'displej',
        'dispozice', 'divadlo', 'divoký', 'divák', 'dlaň', 'dle', 'dlouho', 'dlouhodobý', 'dlouhý', 'dnes', 'dneska',
        'dnešní', 'dno', 'do', 'doba', 'dobrý', 'dobře', 'docela', 'docházet', 'dodat', 'dodnes', 'dodávat', 'dohoda',
        'dohromady', 'dojem', 'dojít', 'dokonalý', 'dokonce', 'dokončit', 'doktor', 'dokud', 'dokument', 'dokázat',
        'dolar', 'dolů', 'doma', 'domnívat', 'domov', 'domácnost', 'domácí', 'domů', 'dopadnout', 'dopis', 'doplnit',
        'doporučovat', 'doprava', 'dopravní', 'dorazit', 'dosahovat', 'doslova', 'dospělý', 'dost', 'dostat',
        'dostatečný', 'dostatečně', 'dostupný', 'dostávat', 'dosud', 'dosáhnout', 'dotace', 'dotknout', 'doufat',
        'dovnitř', 'dovolená', 'dovolit', 'dovést', 'dozvědět', 'dočkat', 'drahý', 'drobný', 'druh', 'druhý', 'dráha',
        'držet', 'duben', 'duch', 'duše', 'dva', 'dvacet', 'dvakrát', 'dvanáct', 'dveře', 'dvůr', 'dále', 'dáma', 'dát',
        'dávat', 'dávka', 'dávno', 'dávný', 'délka', 'déšť', 'díky', 'díl', 'dílo', 'díra', 'dít', 'dítě', 'dívat',
        'dívka', 'dějiny', 'děkovat', 'dělat', 'dětský', 'dětství', 'dřevo', 'dřevěný', 'důkaz', 'důležitý', 'dům',
        'důsledek', 'důvod', 'ekonomický', 'ekonomika', 'elektrický', 'energetický', 'energie', 'euro', 'Evropa',
        'evropský', 'existence', 'existovat', 'fakt', 'faktor', 'fakulta', 'fanoušek', 'festival', 'film', 'filmový',
        'finance', 'finanční', 'firma', 'fond', 'forma', 'fotbal', 'fotbalový', 'fotka', 'fotografie', 'Francie',
        'francouzský', 'fungovat', 'funkce', 'fyzický', 'fáze', 'generace', 'gól', 'hala', 'herec', 'hezký',
        'historický', 'historie', 'hladina', 'hlas', 'hlava', 'hlavní', 'hlavně', 'hledat', 'hledisko', 'hledět',
        'hluboký', 'hmota', 'hmotnost', 'hned', 'hnutí', 'hnědý', 'hodina', 'hodit', 'hodlat', 'hodnocení', 'hodnota',
        'hodně', 'holka', 'hora', 'horký', 'horní', 'hospodářský', 'host', 'hotel', 'hotový', 'hovořit', 'hra', 'hrad',
        'hranice', 'hrdina', 'hrozit', 'hrozně', 'hrát', 'hráč', 'hudba', 'hudební', 'hvězda', 'hřiště', 'i', 'ideální',
        'informace', 'informační', 'informovat', 'instituce', 'internet', 'internetový', 'investice', 'italský', 'jak',
        'jakmile', 'jako', 'jaký', 'jakýkoli', 'jakýsi', 'jaro', 'jasný', 'jasně', 'jazyk', 'jeden', 'jedinec',
        'jediný', 'jednak', 'jednat', 'jednoduchý', 'jednoduše', 'jednotka', 'jednotlivý', 'jednou', 'jednání', 'jeho',
        'jejich', 'její', 'jelikož', 'jemný', 'jen', 'jenom', 'jenž', 'jenže', 'jestli', 'jestliže', 'jet', 'jev',
        'jezdit', 'ještě', 'jinak', 'jinde', 'jiný', 'jistota', 'jistý', 'jistě', 'již', 'jižní', 'jmenovat', 'jméno',
        'jo', 'já', 'jádro', 'jídlo', 'jíst', 'jít', 'jízda', 'k', 'kam', 'kamarád', 'kamenný', 'kamera', 'kancelář',
        'kapacita', 'kapela', 'kapitola', 'kapitán', 'kapsa', 'kariéra', 'karta', 'kategorie', 'každý', 'kde', 'kdo',
        'kdy', 'kdyby', 'kdykoli', 'kdysi', 'když', 'kilometr', 'klasický', 'klid', 'klidný', 'klidně', 'klient',
        'klub', 'kluk', 'klást', 'klíč', 'klíčový', 'kniha', 'knihovna', 'knížka', 'kolega', 'kolem', 'koleno', 'kolik',
        'kolo', 'kombinace', 'komise', 'komora', 'komunikace', 'konat', 'koncert', 'konec', 'konečný', 'konečně',
        'konkrétní', 'konstrukce', 'kontakt', 'kontrola', 'končit', 'kopec', 'koruna', 'kost', 'kostel', 'koupit',
        'kousek', 'kočka', 'košile', 'kraj', 'krajina', 'krajský', 'krev', 'krize', 'krk', 'krok', 'kromě', 'kruh',
        'král', 'krása', 'krásný', 'krátce', 'krátký', 'který', 'kuchyně', 'kultura', 'kulturní', 'kurs', 'kus',
        'kvalita', 'kvalitní', 'květ', 'květen', 'kvůli', 'kámen', 'káva', 'křeslo', 'křičet', 'křídlo', 'kůň', 'kůže',
        'led', 'leden', 'lehce', 'lehký', 'les', 'letadlo', 'letní', 'letos', 'letošní', 'levný', 'levý', 'ležet',
        'lidový', 'lidský', 'liga', 'linka', 'list', 'listopad', 'literatura', 'lišit', 'lokalita', 'Londýn', 'loď',
        'loňský', 'lze', 'láska', 'látka', 'lék', 'lékař', 'léto', 'léčba', 'líbit', 'majetek', 'majitel', 'malý',
        'maminka', 'manažer', 'manžel', 'manželka', 'mapa', 'maso', 'materiál', 'matka', 'metoda', 'metr', 'mezi',
        'mezinárodní', 'miliarda', 'milimetr', 'milión', 'milovat', 'milý', 'mimo', 'ministerstvo', 'ministr',
        'minulost', 'minulý', 'minuta', 'mistr', 'mladík', 'mladý', 'mluvit', 'mluvčí', 'mléko', 'mnohem', 'mnoho',
        'mnohý', 'množství', 'mobil', 'mobilní', 'moc', 'moci', 'model', 'moderní', 'modrý', 'moment', 'Morava', 'most',
        'motor', 'mozek', 'moře', 'možnost', 'možná', 'možný', 'mrtvý', 'muset', 'muzeum', 'muž', 'my', 'mysl',
        'myslet', 'myšlenka', 'málo', 'máma', 'médium', 'míra', 'mírně', 'místnost', 'místní', 'místo', 'mít', 'měnit',
        'město', 'městský', 'měsíc', 'můj', 'na', 'nabídka', 'nabídnout', 'nabízet', 'nacházet', 'nad', 'nadále',
        'naděje', 'nahoru', 'nahradit', 'najednou', 'najít', 'nakonec', 'nalézt', 'naopak', 'napadnout', 'naposledy',
        'naprosto', 'napsat', 'napětí', 'například', 'narazit', 'narodit', 'nastat', 'nastoupit', 'natolik', 'naučit',
        'navrhnout', 'navzdory', 'navíc', 'navštívit', 'nazývat', 'naštěstí', 'ne', 'nebe', 'nebezpečí', 'nebo',
        'neboť', 'nechat', 'nechávat', 'nedostatek', 'nedávno', 'neděle', 'nehoda', 'nejen', 'nejprve', 'nemoc',
        'nemocnice', 'nemocný', 'nepřítel', 'neustále', 'nezbytný', 'než', 'nic', 'nicméně', 'nijak', 'nikdo', 'nikdy',
        'nikoli', 'no', 'noc', 'noha', 'norma', 'normální', 'nos', 'nosit', 'novinka', 'noviny', 'novinář', 'nový',
        'nově', 'noční', 'nutit', 'nutný', 'nyní', 'nábytek', 'nádherný', 'náhle', 'náhodou', 'náklad', 'nákup',
        'nálada', 'náměstí', 'nápad', 'národ', 'národní', 'nárok', 'náročný', 'následek', 'následně', 'následovat',
        'následující', 'nástroj', 'návrat', 'návrh', 'návštěva', 'návštěvník', 'název', 'názor', 'náš', 'nést', 'nízký',
        'nýbrž', 'něco', 'nějak', 'nějaký', 'někde', 'někdo', 'někdy', 'několik', 'několikrát', 'některý', 'Němec',
        'Německo', 'německý', 'o', 'oba', 'obava', 'obchod', 'obchodní', 'období', 'obec', 'obecný', 'obecně', 'objekt',
        'objem', 'objevit', 'objevovat', 'oblast', 'oblečení', 'obličej', 'oblíbený', 'obor', 'obr', 'obrana', 'obraz',
        'obrovský', 'obrátit', 'obrázek', 'obsah', 'obsahovat', 'obvod', 'obvykle', 'obvyklý', 'obyvatel', 'obyčejný',
        'občan', 'občanský', 'občas', 'oběť', 'ochrana', 'ocitnout', 'od', 'odborník', 'odborný', 'odchod', 'odcházet',
        'oddělení', 'odejít', 'odhalit', 'odjet', 'odkud', 'odlišný', 'odmítat', 'odmítnout', 'odpoledne', 'odpor',
        'odpovídat', 'odpovědět', 'odpověď', 'oheň', 'ohled', 'okamžik', 'okamžitě', 'okno', 'oko', 'okolnost',
        'okolní', 'okolo', 'okolí', 'okraj', 'olej', 'omezený', 'on', 'onemocnění', 'onen', 'oni', 'opakovat',
        'opatření', 'operace', 'operační', 'oprava', 'opravdu', 'oproti', 'opustit', 'opět', 'organizace', 'orgán',
        'osm', 'osoba', 'osobnost', 'osobní', 'osobně', 'ostatní', 'ostatně', 'Ostrava', 'ostrov', 'ostrý', 'osud',
        'otec', 'otevřený', 'otevřít', 'otočit', 'otázka', 'ovlivnit', 'ovšem', 'označit', 'označovat', 'oznámit',
        'ozvat', 'očekávat', 'pacient', 'padat', 'padesát', 'padnout', 'pak', 'pamatovat', 'památka', 'paměť', 'pan',
        'paní', 'papír', 'parametr', 'park', 'partner', 'patnáct', 'patro', 'patřit', 'paže', 'peníze', 'pes', 'pevný',
        'pevně', 'pivo', 'planeta', 'platit', 'plný', 'plně', 'plocha', 'plyn', 'Plzeň', 'plán', 'plánovat', 'po',
        'pobyt', 'pochopit', 'pochopitelně', 'pocházet', 'pocit', 'pod', 'podat', 'podařit', 'podepsat', 'podivný',
        'podlaha', 'podle', 'podmínka', 'podnik', 'podoba', 'podobný', 'podobně', 'podpora', 'podporovat', 'podpořit',
        'podstata', 'podstatný', 'podzim', 'podávat', 'podíl', 'podílet', 'podívat', 'pohled', 'pohlédnout', 'pohyb',
        'pohybovat', 'pojem', 'pokaždé', 'pokoj', 'pokoušet', 'pokračovat', 'pokud', 'pokus', 'pokusit', 'pole',
        'policejní', 'policie', 'policista', 'politický', 'politik', 'politika', 'poloha', 'polovina', 'položit',
        'pomalu', 'pomoc', 'pomoci', 'pomocí', 'pomyslet', 'pomáhat', 'poměr', 'poměrně', 'poněkud', 'popis',
        'popisovat', 'poprvé', 'popsat', 'populace', 'poradit', 'posadit', 'poskytnout', 'poskytovat', 'poslat',
        'poslední', 'poslouchat', 'postava', 'postavení', 'postavit', 'postel', 'postoj', 'postup', 'postupně',
        'potkat', 'potom', 'potravina', 'potvrdit', 'poté', 'potíž', 'potřeba', 'potřebný', 'potřebovat', 'pouhý',
        'pouze', 'použití', 'použít', 'používat', 'povaha', 'považovat', 'povinnost', 'povrch', 'povést', 'povídat',
        'povědět', 'pozdní', 'pozdě', 'pozemek', 'pozice', 'pozitivní', 'poznamenat', 'poznat', 'poznámka', 'pozor',
        'pozornost', 'pozorovat', 'pozvat', 'počasí', 'počet', 'počkat', 'počátek', 'počítat', 'počítač', 'pořád',
        'pořádek', 'pořádně', 'pořídit', 'požadavek', 'požádat', 'prach', 'pracovat', 'pracovní', 'pracovník', 'Praha',
        'prakticky', 'praktický', 'pravda', 'pravděpodobně', 'pravidelný', 'pravidelně', 'pravidlo', 'pravý', 'praxe',
        'pražský', 'premiér', 'prezident', 'princip', 'pro', 'problém', 'probudit', 'probíhat', 'proběhnout',
        'procento', 'proces', 'procházet', 'prodat', 'prodej', 'produkce', 'produkt', 'prodávat', 'profesor', 'program',
        'prohlásit', 'projekt', 'projev', 'projevit', 'projevovat', 'projít', 'promluvit', 'proměnit', 'prosinec',
        'prosit', 'prostor', 'prostě', 'prostředek', 'prostřednictvím', 'prostředí', 'proti', 'proto', 'protože',
        'proud', 'provedení', 'provoz', 'provádět', 'provést', 'prozradit', 'proč', 'prst', 'prvek', 'první', 'pryč',
        'práce', 'právní', 'právo', 'právě', 'prázdný', 'prý', 'průběh', 'průmysl', 'průměr', 'průměrný', 'psát',
        'pták', 'ptát', 'pustit', 'pád', 'pán', 'pár', 'pátek', 'péče', 'píseň', 'pít', 'pěkný', 'pěkně', 'pět',
        'přece', 'před', 'předchozí', 'předem', 'především', 'předmět', 'přednost', 'přední', 'předpoklad',
        'předpokládat', 'předseda', 'představa', 'představení', 'představit', 'představovat', 'předtím', 'přejít',
        'překvapit', 'přemýšlet', 'přes', 'přesný', 'přesně', 'přestat', 'přesto', 'přestože', 'přesvědčit', 'převzít',
        'přečíst', 'přežít', 'při', 'přibližně', 'přiblížit', 'přicházet', 'přidat', 'přijet', 'přijmout', 'přijít',
        'přikývnout', 'přinášet', 'přinést', 'připadat', 'připojit', 'připomenout', 'připomínat', 'připravený',
        'připravit', 'připravovat', 'přirozený', 'přitom', 'přivést', 'přiznat', 'přičemž', 'přání', 'přát', 'příběh',
        'příjem', 'příjemný', 'příklad', 'příležitost', 'příliš', 'přímo', 'přímý', 'případ', 'případný', 'případně',
        'příprava', 'příroda', 'přírodní', 'příslušný', 'příspěvek', 'přístroj', 'přístup', 'přítel', 'přítomnost',
        'přítomný', 'příčina', 'příští', 'půda', 'půl', 'působení', 'působit', 'původ', 'původní', 'původně', 'rada',
        'radnice', 'radost', 'rameno', 'reagovat', 'reakce', 'realita', 'realizace', 'region', 'regionální',
        'rekonstrukce', 'republika', 'restaurace', 'ret', 'reálný', 'režim', 'režisér', 'riziko', 'rodina', 'rodinný',
        'rodič', 'roh', 'rok', 'role', 'román', 'rostlina', 'rovnice', 'rovnou', 'rovněž', 'rozdíl', 'rozdělit',
        'rozhodnout', 'rozhodnutí', 'rozhodně', 'rozhodovat', 'rozhovor', 'rozměr', 'rozpočet', 'rozsah', 'rozsáhlý',
        'rozumět', 'rozvoj', 'rozšířit', 'ročník', 'ruka', 'Rusko', 'ruský', 'ryba', 'rychle', 'rychlost', 'rychlý',
        'rád', 'rámec', 'rána', 'ráno', 'růst', 'různý', 's', 'samostatný', 'samotný', 'samozřejmě', 'samý', 'sbor',
        'sbírka', 'schod', 'schopnost', 'schopný', 'scéna', 'sdružení', 'sdělit', 'se', 'sedm', 'sednout', 'sedět',
        'sejít', 'sem', 'sen', 'seriál', 'sestra', 'setkat', 'setkání', 'severní', 'seznam', 'seznámit', 'sezona',
        'sice', 'signál', 'silnice', 'silný', 'silně', 'situace', 'skladba', 'sklo', 'skončit', 'skoro', 'skrývat',
        'skupina', 'skutečnost', 'skutečný', 'skutečně', 'skvělý', 'skála', 'slabý', 'slavný', 'sledovat', 'slečna',
        'sloužit', 'Slovensko', 'slovenský', 'slovo', 'složitý', 'složka', 'slunce', 'sluneční', 'služba', 'slyšet',
        'slza', 'smlouva', 'smrt', 'smysl', 'smát', 'smích', 'směr', 'smět', 'snad', 'snadno', 'snadný', 'snaha',
        'snažit', 'sníh', 'snímek', 'snížit', 'sobota', 'sociální', 'sotva', 'soubor', 'soud', 'souhlasit', 'soukromý',
        'soupeř', 'soused', 'soustava', 'soustředit', 'soutěž', 'souviset', 'souvislost', 'současnost', 'současný',
        'současně', 'součást', 'spadnout', 'spatřit', 'specifický', 'speciální', 'spisovatel', 'splnit', 'spodní',
        'spojení', 'spojený', 'spojit', 'spokojený', 'společenský', 'společnost', 'společný', 'společně', 'spolu',
        'spolupráce', 'spor', 'sport', 'sportovní', 'spotřeba', 'spousta', 'spočívat', 'správa', 'správný', 'správně',
        'spustit', 'spánek', 'spát', 'spíš', 'srdce', 'srovnání', 'srpen', 'stanice', 'stanovit', 'starat', 'starost',
        'starosta', 'starý', 'stav', 'stavba', 'stavební', 'stavět', 'stačit', 'stejný', 'stejně', 'stihnout', 'sto',
        'století', 'stopa', 'stovka', 'strach', 'strana', 'strategie', 'strašně', 'stroj', 'strom', 'struktura',
        'stránka', 'strávit', 'student', 'studený', 'studie', 'studium', 'studovat', 'stupeň', 'styl', 'stáhnout',
        'stále', 'stát', 'státní', 'stávat', 'stín', 'stěna', 'střecha', 'střední', 'stůl', 'suchý', 'svatý', 'svaz',
        'svoboda', 'svobodný', 'svět', 'světlo', 'světový', 'svůj', 'symbol', 'syn', 'systém', 'sál', 'sám', 'série',
        'síla', 'síť', 'sůl', 'tabulka', 'tady', 'tajemství', 'tajný', 'tak', 'takhle', 'takový', 'takto', 'taky',
        'takzvaný', 'také', 'takže', 'tam', 'technický', 'technika', 'technologie', 'teda', 'tedy', 'tehdejší', 'tehdy',
        'telefon', 'televize', 'televizní', 'temný', 'ten', 'tenhle', 'tenkrát', 'tento', 'tentokrát', 'tentýž',
        'teorie', 'teplo', 'teplota', 'teplý', 'teprve', 'termín', 'test', 'text', 'teď', 'ticho', 'tichý', 'tisíc',
        'titul', 'tiše', 'tlak', 'tlačítko', 'tma', 'tmavý', 'to', 'tolik', 'totiž', 'touha', 'toužit', 'tradice',
        'tradiční', 'trasa', 'trať', 'trend', 'trenér', 'trest', 'trh', 'trochu', 'trpět', 'trvat', 'tráva', 'tu',
        'turnaj', 'tušit', 'tvar', 'tvorba', 'tvořit', 'tvrdit', 'tvrdý', 'tvář', 'tvůj', 'ty', 'typ', 'typický',
        'tábor', 'táhnout', 'táta', 'téma', 'téměř', 'též', 'tón', 'týden', 'týkat', 'tým', 'týž', 'tělo', 'těsně',
        'těšit', 'těžko', 'těžký', 'třeba', 'třetina', 'třetí', 'tři', 'třicet', 'třída', 'u', 'ucho', 'udržet',
        'udržovat', 'událost', 'udělat', 'ukazovat', 'ukázat', 'ulice', 'uložit', 'umožnit', 'umožňovat', 'umístit',
        'umělec', 'umělecký', 'umělý', 'umění', 'umět', 'unie', 'univerzita', 'upozornit', 'uprostřed', 'určený',
        'určit', 'určitý', 'určitě', 'uskutečnit', 'usmát', 'usmívat', 'utkání', 'utéci', 'uvažovat', 'uvedený',
        'uvidět', 'uvnitř', 'uvádět', 'uvést', 'uvědomit', 'uvědomovat', 'uzavřít', 'učinit', 'učit', 'učitel', 'už',
        'uživatel', 'užívat', 'v', 'vadit', 'varianta', 'vazba', 'vedení', 'vedle', 'vedoucí', 'vejít', 'velice',
        'velikost', 'veliký', 'velký', 'velmi', 'ven', 'venku', 'verze', 'vesmír', 'vesnice', 'večer', 'večeře',
        'veřejnost', 'veřejný', 'veškerý', 'vhodný', 'viditelný', 'vidět', 'vina', 'viset', 'viz', 'vlak', 'vlas',
        'vlastnost', 'vlastní', 'vlastně', 'vliv', 'vlna', 'vloni', 'vláda', 'vnitřní', 'vnímat', 'vnější', 'voda',
        'vodní', 'vojenský', 'voják', 'volat', 'volba', 'volit', 'volný', 'vozidlo', 'vracet', 'vrchol', 'vrstva',
        'vrátit', 'vstoupit', 'vstup', 'vstát', 'vteřina', 'vy', 'vybavit', 'vybraný', 'vybrat', 'vybírat', 'vycházet',
        'vydat', 'vydržet', 'vydání', 'vydávat', 'vyhnout', 'vyhrát', 'vyjádřit', 'vyjít', 'vypadat', 'vyprávět',
        'vyrazit', 'vyrábět', 'vyskytovat', 'vysoko', 'vysoký', 'vystoupit', 'vystupovat', 'vysvětlit', 'vysvětlovat',
        'vytvořit', 'vytvářet', 'vytáhnout', 'využití', 'využít', 'využívat', 'vyvolat', 'vyzkoušet', 'vyřešit',
        'vyžadovat', 'vzduch', 'vzdálenost', 'vzdálený', 'vzdát', 'vzdělání', 'vzdělávání', 'vzhledem', 'vznik',
        'vznikat', 'vzniknout', 'vzor', 'vzpomenout', 'vzpomínat', 'vzpomínka', 'vztah', 'vzájemný', 'vzít', 'váha',
        'válka', 'Vánoce', 'vánoční', 'váš', 'vážný', 'vážně', 'vést', 'víc', 'více', 'víkend', 'víno', 'víra', 'vítr',
        'vítěz', 'vítězství', 'výbor', 'výběr', 'východ', 'východní', 'výchova', 'výhoda', 'výjimka', 'výkon', 'výměna',
        'výraz', 'výrazný', 'výrazně', 'výroba', 'výrobce', 'výrobek', 'výsledek', 'výstava', 'výstavba', 'vývoj',
        'výzkum', 'význam', 'významný', 'výzva', 'výše', 'výška', 'včera', 'včetně', 'věc', 'věda', 'vědec', 'vědecký',
        'vědomí', 'vědět', 'věk', 'věnovat', 'věta', 'větev', 'většina', 'většinou', 'vězení', 'věřit', 'věž', 'však',
        'všechen', 'všimnout', 'všude', 'vůbec', 'vůle', 'vůně', 'vůz', 'vůči', 'vždy', 'vždycky', 'vždyť', 'z', 'za',
        'zabránit', 'zabít', 'zabývat', 'zachovat', 'zachránit', 'zadní', 'zahrada', 'zahraniční', 'zahraničí',
        'zahájit', 'zajistit', 'zajímat', 'zajímavý', 'zajít', 'zakázka', 'založit', 'zamířit', 'zaměstnanec',
        'zaměřit', 'zaplatit', 'zapomenout', 'zas', 'zase', 'zasmát', 'zastavit', 'zasáhnout', 'zatím', 'zatímco',
        'zaujmout', 'zavolat', 'zavést', 'zavřít', 'zaznamenat', 'začátek', 'začínat', 'začít', 'zařízení', 'zažít',
        'zbavit', 'zboží', 'zbraň', 'zbytek', 'zbýt', 'zbývat', 'zcela', 'zda', 'zde', 'zdravotní', 'zdraví', 'zdravý',
        'zdroj', 'zdát', 'zejména', 'zelený', 'země', 'zemřít', 'zeptat', 'zeď', 'zhruba', 'zima', 'zimní', 'zisk',
        'zjistit', 'zkouška', 'zkrátka', 'zkusit', 'zkušenost', 'zlato', 'zlatý', 'zlý', 'zmizet', 'zmínit', 'zmíněný',
        'změna', 'změnit', 'znak', 'znalost', 'znamenat', 'značka', 'značný', 'znovu', 'známý', 'znát', 'znít',
        'zpravidla', 'zpráva', 'zpátky', 'zpívat', 'zpět', 'způsob', 'způsobit', 'zrovna', 'ztratit', 'ztrácet',
        'ztráta', 'zub', 'zvednout', 'zvládnout', 'zvláštní', 'zvláště', 'zvlášť', 'zvolit', 'zvuk', 'zvyšovat',
        'zvíře', 'zvýšení', 'zvýšit', 'záda', 'zájem', 'zákazník', 'základ', 'základní', 'zákon', 'záležet',
        'záležitost', 'zámek', 'západ', 'západní', 'zápas', 'zároveň', 'zásada', 'zásadní', 'zásah', 'zástupce',
        'závislost', 'závislý', 'závod', 'závěr', 'záznam', 'září', 'zážitek', 'získat', 'zítra', 'zřejmě', 'zůstat',
        'zůstávat', 'údaj', 'úkol', 'únor', 'úplný', 'úplně', 'úprava', 'úroveň', 'úsek', 'úsměv', 'úspěch', 'úspěšný',
        'ústa', 'ústav', 'útok', 'útočník', 'úvaha', 'území', 'úzký', 'účast', 'účastník', 'účel', 'účet', 'účinek',
        'úřad', 'úžasný', 'čaj', 'čas', 'časopis', 'časový', 'často', 'častý', 'Čech', 'Čechy', 'čekat', 'čelo',
        'černý', 'čerstvý', 'červen', 'červenec', 'červený', 'Česko', 'český', 'či', 'čin', 'činit', 'činnost', 'čistý',
        'člen', 'člověk', 'článek', 'čtenář', 'čtvrtý', 'čtyři', 'část', 'částice', 'částka', 'Čína', 'čínský', 'číslo',
        'číst', 'řada', 'ředitel', 'řeka', 'řeč', 'řešení', 'řešit', 'řidič', 'řád', 'říci', 'řídit', 'říjen', 'říkat',
        'řízení', 'šance', 'šaty', 'šedý', 'šest', 'široký', 'škoda', 'škola', 'školní', 'špatný', 'špatně', 'štěstí',
        'šéf', 'šťastný', 'že', 'žena', 'ženský', 'židle', 'život', 'životní', 'živý', 'žlutý', 'žádat', 'žádný',
        'žádost', 'žák', 'žít',
    )



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    address_formats = ['{{street_address}}, {{city}}, {{postcode}}']
    building_number_formats = ['#', '##', '###']
    city_formats = ['{{city_prefix}} {{first_name}}']
    street_address_formats = ['{{street_name}}, {{building_number}}']
    street_name_formats = ('{{street_prefix}} {{street_title}}', )

    city_prefixes = ['місто', 'село', 'селище', 'хутір']
    countries = [
        'Австралія', 'Австрія', 'Азербайджан', 'Албанія', 'Алжир', 'Ангола',
        'Андорра', 'Антигуа і Барбуда', 'Аргентина', 'Афганістан',
        'Багамські Острови', 'Бангладеш', 'Барбадос', 'Бахрейн', 'Беліз',
        'Бельгія', 'Бенін', 'Білорусь', 'Болгарія', 'Болівія',
        'Боснія і Герцеговина', 'Ботсвана', 'Бразилія', 'Бруней',
        'Буркіна-Фасо', 'Бурунді', 'Бутан', 'Вануату', 'Ватикан',
        'Велика Британія', 'Венесуела', 'В\'єтнам', 'Вірменія', 'Габон',
        'Гаїті', 'Гаяна', 'Гамбія', 'Гана', 'Гватемала', 'Гвінея',
        'Гвінея-Бісау', 'Гондурас', 'Гренада', 'Греція', 'Грузія', 'Данія',
        'Джибуті', 'Домініка', 'Домініканська Республіка', 'Еквадор',
        'Екваторіальна Гвінея', 'Еритрея', 'Естонія', 'Ефіопія', 'Єгипет',
        'Ємен', 'Замбія', 'Західна Сахара', 'Зімбабве', 'Ізраїль', 'Індія',
        'Індонезія', 'Ірак', 'Іран', 'Ірландія', 'Ісландія', 'Іспанія',
        'Італія', 'Йорданія', 'Кабо-Верде', 'Казахстан', 'Камбоджа', 'Камерун',
        'Канада', 'Катар', 'Кенія', 'Киргизстан', 'КНР', 'Кіпр', 'Кірибаті',
        'Колумбія', 'Коморські Острови', 'Конго', 'ДР Конго', 'Південна Корея',
        'Північна Корея', 'Косово', 'Коста-Рика', 'Кот-д\'Івуар', 'Куба',
        'Кувейт', 'Лаос', 'Латвія', 'Лесото', 'Литва', 'Ліберія', 'Ліван',
        'Лівія', 'Ліхтенштейн', 'Люксембург', 'Маврикій', 'Мавританія',
        'Мадагаскар', 'Республіка Македонія', 'Малаві', 'Малайзія', 'Малі',
        'Мальдіви', 'Мальта', 'Марокко', 'Маршаллові Острови', 'Мексика',
        'Федеративні Штати Мікронезії', 'Мозамбік', 'Молдова', 'Монако',
        'Монголія', 'М\'янма', 'Намібія', 'Науру', 'Непал', 'Нігер', 'Нігерія',
        'Нідерланди', 'Нікарагуа', 'Німеччина', 'Нова Зеландія', 'Норвегія',
        'ОАЕ', 'Оман', 'Пакистан', 'Палау', 'Палестинська держава', 'Панама',
        'Папуа Нова Гвінея', 'ПАР', 'Парагвай', 'Перу', 'Південний Судан',
        'Польща', 'Португалія', 'Росія', 'Руанда', 'Румунія', 'Сальвадор',
        'Самоа', 'Сан-Марино', 'Сан-Томе і Принсіпі', 'Саудівська Аравія',
        'Свазіленд', 'Сейшельські Острови', 'Сенегал',
        'Сент-Вінсент і Гренадини', 'Сент-Кіттс і Невіс', 'Сент-Люсія',
        'Сербія', 'Сінгапур', 'Сирія', 'Словаччина', 'Словенія',
        'Соломонові Острови', 'Сомалі', 'Судан', 'Суринам', 'Східний Тимор',
        'США', 'Сьєрра-Леоне', 'Таджикистан', 'Таїланд', 'Тайвань', 'Танзанія',
        'Того', 'Тонга', 'Тринідад і Тобаго', 'Тувалу', 'Туніс', 'Туреччина',
        'Туркменістан', 'Уганда', 'Угорщина', 'Узбекистан', 'Україна',
        'Уругвай', 'Фіджі', 'Філіппіни', 'Фінляндія', 'Франція', 'Хорватія',
        'Центральноафриканська Республіка', 'Чад', 'Чехія', 'Чилі',
        'Чорногорія', 'Швейцарія', 'Швеція', 'Шрі-Ланка', 'Ямайка', 'Японія',
    ]
    street_prefixes = [
        'вулиця', 'набережна',
    ]
    street_suffixes = ['узвіз']

    # https://data.gov.ua/en/dataset/__-120/resource/76e2684f-e75c-45e5-a0b2-6d5596ab47f8
    # https://opendata.drohobych-rada.gov.ua/sites/default/files/%D0%90%D0%B4%D1%80%D0%B5%D1%81%D0%B8%20%D0%94%D1%80%D0%BE%D0%B3%D0%BE%D0%B1%D0%B8%D1%87.xlsx
    street_titles = [
        '22 Січня',
        'Адама Міцкевича',
        'Академіка Сахарова',
        'Андрея Шептицького',
        'Андрія Мельника',
        'Андрія Чайковського',
        'Антона Чехова',
        'Архипа Люльки',
        'Байди',
        'Богдана Лепкого',
        'Богдана Хмельницького',
        'Бойківська',
        'Бориса Грінченка',
        'Бориславська',
        'Братів Кицилів',
        'Броніслава Козловського',
        'Бруно Шульца',
        'Будівельна',
        'Буковинська',
        "В'ячеслава Чорновола",
        'Василя Барського',
        'Василя Симоненка',
        'Василя Стуса',
        'Вербова',
        'Вокзальна',
        'Володимира Івасюка',
        'Володимира Бірчака',
        'Володимира Великого',
        'Володимира Вернадського',
        'Володимира Винниченка',
        'Володимира Гнатюка',
        'Володимира Чапельського',
        'Володимира Чапельського',
        'Гайдамацька',
        'Героїв Крут',
        'Глінки',
        'Гончарська',
        'Горішня Брама',
        'Григорія Коссака',
        'Григорія Сковороди',
        'Грюнвальдська',
        'Данила Галицького',
        'Данила Нечая',
        'Дмитра Бортнянського',
        'Дмитра Бурка',
        'Дмитра Вітовського',
        'Дмитра Грицая',
        'Дмитра Котка',
        'Дністровська',
        'Євгена Гребінки',
        'Євгена Коновальця',
        'Захарії Копистенського',
        'Зварицька',
        'Зелена',
        'Івана Вагилевича',
        'Івана Вендзиловича',
        'Івана Верхратського',
        'Івана Карпенка-Карого',
        'Івана Котляревського',
        'Івана Мазепи',
        'Івана Макуха',
        'Івана Пулюя',
        'Івана Підкови',
        'Івана Сірка',
        'Івана Труша',
        'Івана Франка',
        'Іллі Рєпіна',
        'Індустріальна',
        'Ірини Вільде',
        'Йосипа Левицького',
        'Йосипа Сліпого',
        'Йосифа Стадника',
        'Карпатська',
        'Кирило - Мефодіївська',
        'Київська',
        'Княгині Ольги',
        'Ковальська',
        'Козацька',
        'Коминярська',
        'Крушельницької',
        'Купальська',
        'Лева Толстого',
        'Лемківська',
        'Леся Курбаса',
        'Леся Мартовича',
        'Лесі Українки',
        'Львівська',
        'Людвіга ван Бетховена',
        'Лішнянська',
        'Максима Залізняка',
        'Максима Кривоноса',
        'Максима Рильського',
        'Малий Ринок',
        'Марка Кропивницького',
        'Маркіяна Шашкевича',
        'Марії Заньковецької',
        'Маріїнська',
        'Механізаторів',
        'Миколи Вороного',
        'Миколи Гоголя',
        'Миколи Леонтовича',
        'Миколи Лисенка',
        'Миколи Некрасова',
        'Мирона Тарнавського',
        'Мирослава Тураша',
        'Михайла Вербицького',
        'Михайла Грушевського',
        'Михайла Зубрицького',
        'Михайла Коцюбинського',
        'Михайла Лермонтова',
        'Михайла Остроградського',
        'Михайла Павлика',
        'Михайла Старицького',
        'Млинарська',
        'Модеста Менцинського',
        'Музична',
        'Нафтовиків',
        'Нова',
        'Озерна',
        'Олега Ольжича',
        'Олександра Кониського',
        'Олександра Пушкіна',
        'Олекси Довбуша',
        'Олексндра Довженка',
        'Олени Теліги',
        'Олеся Гончара',
        'Опришківська',
        'Осипа Турянського',
        'Остапа Вишні',
        'Остапа Нижанківського',
        'Павла Грабовського',
        'Павла Полуботка',
        'Панаса Мирного',
        'Паркова',
        'Петра Дорошенка',
        'Петра Сагайдачного',
        'Пилипа Орлика',
        'Пластунів',
        'Прорізна',
        'Південна',
        'Раневицька',
        'Рихтицька',
        'Рівна',
        'Різдвяна',
        'Ріпницька',
        'Самбірська',
        'Северина Наливайка',
        'Солоний Ставок',
        'Спартака',
        'Спортивна',
        'Станіслава Людкевича',
        'Станіславська',
        'Степана Коваліва',
        'Степана Рудавського',
        'Стрийська',
        'Східна',
        'Січових Стрільців',
        'Тараса Шевченка',
        'Твердохліба',
        'Тисменицька',
        'Тита Реваковича',
        'Тиха',
        'Трускавецька',
        'Уляни Кравченко',
        'Устима Кармелюка',
        'Устияновичів',
        'Фабрична',
        'Фредеріка Шопена',
        'Холмська',
        'Шевська',
        'Шкільна',
        'Шолом Алейхема',
        'Шота Руставелі',
        'Юліуша Словацького',
        'Юрія Дрогобича',
        'Юрія Федьковича',
        'Якова Головацького',
        'Ярослава Осмомисла',
    ]

    def city_prefix(self):
        return self.random_element(self.city_prefixes)

    def postcode(self):
        """The code consists of five digits (01000-99999)"""
        return f'{self.generator.random.randint(0, 10)}{self.generator.random.randint(1000, 10000)}'

    def street_prefix(self):
        return self.random_element(self.street_prefixes)

    def street_title(self):
        return self.random_element(self.street_titles)



from faker.providers.bank import Provider as BankProvider


class Provider(BankProvider):
    """Implement bank provider for ``ro_RO`` locale."""

    country_code = 'RO'
    bban_format = '????################'
    swift_bank_codes = (
        'NBOR', 'ABNA', 'BUCU', 'ARBL', 'MIND', 'BPOS', 'CARP', 'RNCB',
        'BROM', 'BITR', 'BRDE', 'BRMA', 'BTRL', 'DAFB', 'MIRB', 'CECE',
        'CITI', 'CRCO', 'FNNB', 'EGNA', 'BSEA', 'EXIM', 'UGBI', 'HVBL',
        'INGB', 'BREL', 'CRDZ', 'BNRB', 'PIRB', 'PORL', 'MIRO', 'RZBL',
        'RZBR', 'ROIN', 'WBAN', 'TRFD', 'TREZ', 'BACX', 'VBBU', 'DARO',
    )



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):

    formats = (
        '{{last_name}} {{company_suffix}}',
        '{{last_name}} {{last_name}} {{company_suffix}}',
        '{{last_name}}',
    )

    company_suffixes = (
        'SRL', 'SA', 'SCA', 'SNC', 'SCS', 'AFJ', 'ASF', 'CON', 'CRL', 'INC',
        'LOC', 'OC1', 'OC2', 'OC3', 'PFA', 'RA', 'SCS', 'SPI', 'URL',
    )

    def company_suffix(self):
        return self.random_element(self.company_suffixes)



import inspect
import warnings


class Documentor:

    def __init__(self, generator):
        """
        :param generator: a localized Generator with providers filled,
                          for which to write the documentation
        :type generator: faker.Generator()
        """
        self.generator = generator
        self.max_name_len = 0
        self.already_generated = []

    def get_formatters(self, locale=None, excludes=None, **kwargs):
        self.max_name_len = 0
        self.already_generated = [] if excludes is None else excludes[:]
        formatters = []
        providers = self.generator.get_providers()
        for provider in providers[::-1]:  # reverse
            if locale and provider.__lang__ != locale:
                continue
            formatters.append(
                (provider, self.get_provider_formatters(provider, **kwargs)),
            )
        return formatters

    def get_provider_formatters(self, provider, prefix='fake.',
                                with_args=True, with_defaults=True):

        formatters = {}

        for name, method in inspect.getmembers(provider, inspect.ismethod):
            # skip 'private' method and inherited methods
            if name.startswith('_') or name in self.already_generated:
                continue

            arguments = []
            faker_args = []
            faker_kwargs = {}

            if name == 'binary':
                faker_kwargs['length'] = 1024
            elif name in ['zip', 'tar']:
                faker_kwargs.update({
                    'uncompressed_size': 1024,
                    'min_file_size': 512,
                })

            if with_args:
                # retrieve all parameter
                argspec = inspect.getfullargspec(method)

                lst = [x for x in argspec.args if x not in ['self', 'cls']]
                for i, arg in enumerate(lst):

                    if argspec.defaults and with_defaults:

                        try:
                            default = argspec.defaults[i]
                            if isinstance(default, str):
                                default = repr(default)
                            else:
                                # TODO check default type
                                default = f"{default}"

                            arg = f'{arg}={default}'

                        except IndexError:
                            pass

                    arguments.append(arg)
                    if with_args == 'first':
                        break

                if with_args != 'first':
                    if argspec.varargs:
                        arguments.append('*' + argspec.varargs)
                    if argspec.varkw:
                        arguments.append('**' + argspec.varkw)

            # build fake method signature
            signature = f"{prefix}{name}({', '.join(arguments)})"

            try:
                # make a fake example
                example = self.generator.format(name, *faker_args, **faker_kwargs)
            except (AttributeError, ValueError) as e:
                warnings.warn(str(e))
                continue
            formatters[signature] = example

            self.max_name_len = max(self.max_name_len, len(signature))
            self.already_generated.append(name)

        return formatters

    @staticmethod
    def get_provider_name(provider_class):
        return provider_class.__provider__



from .. import Provider as InternetProvider


class Provider(InternetProvider):

    user_name_formats = (
        '{{last_name_female}}.{{first_name_female}}',
        '{{last_name_female}}.{{first_name_female}}',
        '{{last_name_male}}.{{first_name_male}}',
        '{{last_name_male}}.{{first_name_male}}',
        '{{first_name_female}}.{{last_name_female}}',
        '{{first_name_male}}.{{last_name_male}}',
        '{{first_name}}##',
        '?{{last_name}}',
        '?{{last_name}}',
        '?{{last_name}}',
    )

    email_formats = ('{{user_name}}@{{free_email_domain}}', )

    free_email_domains = (
        'email.ro',
        'gmail.com',
        'kappa.ro',
        'acasa.ro',
        'zzup.ro',
        '141.ro',
        'post.ro',
    )

    tlds = ('ro', 'com', 'ro')
